package com.zhentao.groups.service.impl;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhentao.groups.dto.AddGroupMembers;
import com.zhentao.groups.dto.AddGroupsDto;
import com.zhentao.groups.dto.DelGroupMembers;
import com.zhentao.groups.dto.GroupDto;
import com.zhentao.groups.mapper.GroupMembersMapper;
import com.zhentao.groups.pojo.GroupMembers;
import com.zhentao.groups.pojo.Groupss;
import com.zhentao.groups.service.GroupMembersService;
import com.zhentao.groups.service.GroupsService;
import com.zhentao.groups.mapper.GroupsMapper;
import com.zhentao.information.cache.GroupMemberCache;
import com.zhentao.shouye.domain.UserShouye;
import com.zhentao.shouye.mapper.UserShouyeMapper;
import com.zhentao.shouye.service.UserShouyeService;
import com.zhentao.vo.Result;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
* @author lzy
* @description 针对表【groups(群组表)】的数据库操作Service实现
* @createDate 2025-06-04 16:00:48
*/
@Service
public class GroupsServiceImpl extends ServiceImpl<GroupsMapper, Groupss>
    implements GroupsService{

    @Autowired
    private GroupMembersMapper groupMembersMapper;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    @Autowired
    private UserShouyeMapper userShouyeMapper;
//  创建群
    @Autowired
    private GroupMembersService groupMembersService;
    @Autowired
    private UserShouyeService userShouyeService;
    @Autowired
    private GroupsMapper groupsMapper;

    @Autowired
    private GroupMemberCache groupMemberCache;

    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class, timeout = 30)
    public Result addGroup(AddGroupsDto addGroupsDto) {
        Groupss groups = new Groupss();
        groups.setGroupId(IdUtil.getSnowflake().nextId());
        groups.setName(addGroupsDto.getName());
        groups.setCreatorId(addGroupsDto.getCreatorId());
        groups.setAvatar(addGroupsDto.getAvatar());
        groups.setAnnouncement(addGroupsDto.getAnnouncement());
        groups.setDescription(addGroupsDto.getDescription());
        groups.setMaxMembers(200);
        groups.setStatus(1);
        groups.setCreatedAt(new Date());
        groups.setUpdatedAt(new Date());
        int insert1 = this.baseMapper.insert(groups);

        GroupMembers groupMembers = new GroupMembers();
        groupMembers.setId(IdUtil.getSnowflake().nextId());
        groupMembers.setGroupId(groups.getGroupId());
        groupMembers.setUserId(groups.getCreatorId());
        groupMembers.setNickname(groups.getName());
        groupMembers.setRole(2);
        groupMembers.setJoinTime(new Date());
        int insert = groupMembersMapper.insert(groupMembers);

        // 更新群成员缓存
        List<Long> memberIds = new ArrayList<>();
        memberIds.add(groups.getCreatorId());
        groupMemberCache.updateGroupMembers(groups.getGroupId(), memberIds);

        UserShouye userShouye=new UserShouye();
        userShouye.setId(IdUtil.getSnowflake().nextId());
        userShouye.setUid1(addGroupsDto.getCreatorId());
        userShouye.setUid2(null);
        userShouye.setGid(groups.getGroupId());
        userShouye.setStatus(0);
        userShouye.setSort(null);
        int insert2 = userShouyeMapper.insert(userShouye);

        if (insert1!=0 && insert!=0 && insert2!=0){
            return Result.OK(null,"创建成功");
        }
        return Result.ERR(null,"创建失败");
    }

//    添加人数
    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class, timeout = 30)
    public Result AddGroupMembers(AddGroupMembers addGroupMembers) {
        List<GroupMembers> data = new ArrayList<>();
        List<UserShouye> userShouyes = new ArrayList<>();
        Groupss groupss = this.getById(addGroupMembers.getGroupssId());
        List<GroupMembers> list = groupMembersService.list(new LambdaQueryWrapper<GroupMembers>().eq(GroupMembers::getGroupId, addGroupMembers.getGroupssId()));

        if (list.size() != 0) {
            if (groupss.getMaxMembers() > list.size()) {
                for (Long groupMembers : addGroupMembers.getUserId()) {
                    GroupMembers groupMembers1 = new GroupMembers();
                    groupMembers1.setId(IdUtil.getSnowflake().nextId());
                    groupMembers1.setGroupId(groupss.getGroupId());
                    groupMembers1.setUserId(groupMembers);
                    groupMembers1.setNickname(groupss.getName());
                    groupMembers1.setRole(0);
                    groupMembers1.setJoinTime(new Date());
                    data.add(groupMembers1);

                    UserShouye userShouye = new UserShouye();
                    userShouye.setId(IdUtil.getSnowflake().nextId());
                    userShouye.setUid1(groupMembers);
                    userShouye.setUid2(null);
                    userShouye.setGid(groupss.getGroupId());
                    userShouye.setStatus(0);
                    userShouye.setSort(null);
                    userShouyes.add(userShouye);
                }
                boolean b1 = groupMembersService.saveBatch(data);
                boolean b = userShouyeService.saveBatch(userShouyes);
                if (b1 && b) {
                    // 更新群成员缓存
                    List<Long> memberIds = list.stream()
                        .map(GroupMembers::getUserId)
                        .collect(Collectors.toList());
                    memberIds.addAll(addGroupMembers.getUserId());
                    groupMemberCache.updateGroupMembers(groupss.getGroupId(), memberIds);
                    return Result.OK(null, "添加成功");
                }
            } else {
                return Result.ERR(null, "群已满");
            }
        } else {
            return Result.ERR(null, "群已不存在");
        }

        return Result.ERR(null, "添加失败");
    }



//     踢人
    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class, timeout = 30)
    public Result DelGroupMembers(DelGroupMembers delGroupMembers) {
        System.err.println(delGroupMembers);
//       用户
        GroupMembers groupMembers = groupMembersMapper.selectOne(new LambdaQueryWrapper<GroupMembers>().eq(GroupMembers::getGroupId, delGroupMembers.getGroupssId()).eq(GroupMembers::getUserId, delGroupMembers.getUId()));
//       被踢掉的人
        GroupMembers groupMembers2 = groupMembersMapper.selectOne(new LambdaQueryWrapper<GroupMembers>().eq(GroupMembers::getUserId, delGroupMembers.getUserId()).eq(GroupMembers::getGroupId, delGroupMembers.getGroupssId()));
        System.err.println(groupMembers2);
        UserShouye userShouye = userShouyeMapper.selectOne(new LambdaQueryWrapper<UserShouye>().eq(UserShouye::getUid1, delGroupMembers.getUserId()).eq(UserShouye::getGid, delGroupMembers.getGroupssId()));
        System.err.println(userShouye);
        if (groupMembers2!=null && groupMembers!=null){
            //        首页展示userShouye表中数据
            if (groupMembers.getRole()==2){
                if (groupMembers2.getRole()==2){
                    return Result.ERR(null,"群主不能踢自己");
                }
                groupMembersMapper.deleteById(groupMembers2);
                userShouyeMapper.deleteById(userShouye);

                // 更新群成员缓存
                List<GroupMembers> remainingMembers = groupMembersService.list(
                    new LambdaQueryWrapper<GroupMembers>()
                        .eq(GroupMembers::getGroupId, delGroupMembers.getGroupssId())
                );
                List<Long> memberIds = remainingMembers.stream()
                    .map(GroupMembers::getUserId)
                    .collect(Collectors.toList());
                groupMemberCache.updateGroupMembers(delGroupMembers.getGroupssId(), memberIds);

                return Result.OK(null,"删除成功");
            }else if (groupMembers.getRole()==1){
                if (groupMembers2.getRole()==0){
                    groupMembersMapper.deleteById(groupMembers2);
                    userShouyeMapper.deleteById(userShouye);

                    // 更新群成员缓存
                    List<GroupMembers> remainingMembers = groupMembersService.list(
                        new LambdaQueryWrapper<GroupMembers>()
                            .eq(GroupMembers::getGroupId, delGroupMembers.getGroupssId())
                    );
                    List<Long> memberIds = remainingMembers.stream()
                        .map(GroupMembers::getUserId)
                        .collect(Collectors.toList());
                    groupMemberCache.updateGroupMembers(delGroupMembers.getGroupssId(), memberIds);

                    return Result.OK(null,"删除成功");
                }else {
                    return Result.ERR(null,"权限不足");
                }
            }else {
                return Result.ERR(null,"权限不足");
            }
        }else {
            return Result.ERR(null,"该用户已经不存在");
        }

    }
//查询所有的群ID和群里面成员ID
    @Override
    public List<GroupDto> getList() {
        List<Groupss> list = this.list();
        List<GroupDto> list3 = new ArrayList<>();
//        所有的群ID
        List<Long> collect = list.stream().map(Groupss::getGroupId).collect(Collectors.toList());
        List<GroupMembers> list1 = groupMembersService.list();
        for (Long c:collect) {
            List<Long> list2 = new ArrayList<>();
            for (GroupMembers m: list1) {
                if (c.equals(m.getGroupId())){
                    list2.add(m.getUserId());
                }
            }
            GroupDto groupDto = new GroupDto();
            groupDto.setGroupId(c);
            groupDto.setUid(list2);
            list3.add(groupDto);
        }
        return list3;
    }


}





